import { Body, Get, JsonController, OnUndefined, Param, Post } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { getConnectionManager, Repository } from 'typeorm';
import { PermissionNotFoundError } from '../errors/PermissionErrors';
import { CreatePermission } from '../models/actions/CreatePermission';
import { Permission } from '../models/entities/Permission';
import { ResponsePermission } from '../models/responses/ResponsePermission';


@JsonController('/permissions')
//@Authorized('RUNNERS:read')
export class PermissionController {
    private permissionRepository: Repository<Permission>;

    /**
     * Gets the repository of this controller's model/entity.
     */
    constructor() {
        this.permissionRepository = getConnectionManager().get().getRepository(Permission);
    }

    @Get()
    @ResponseSchema(ResponsePermission, { isArray: true })
    @OpenAPI({ description: 'Lists all permissions.' })
    async getAll() {
        let responsePermissions: ResponsePermission[] = new Array<ResponsePermission>();
        const permissions = await this.permissionRepository.find({ relations: ['principal'] });
        permissions.forEach(permission => {
            responsePermissions.push(new ResponsePermission(permission));
        });
        return responsePermissions;
    }


    @Get('/:id')
    @ResponseSchema(ResponsePermission)
    @ResponseSchema(PermissionNotFoundError, { statusCode: 404 })
    @OnUndefined(PermissionNotFoundError)
    @OpenAPI({ description: 'Returns a permissions of a specified id (if it exists)' })
    async getOne(@Param('id') id: number) {
        let permission = await this.permissionRepository.findOne({ id: id }, { relations: ['principal'] });
        if (!permission) { throw new PermissionNotFoundError(); }
        return new ResponsePermission(permission);
    }


    @Post()
    @ResponseSchema(ResponsePermission)
    @OpenAPI({ description: 'Create a new runnerTeam object (id will be generated automagicly).' })
    async post(@Body({ validate: true }) createPermission: CreatePermission) {
        let permission;
        try {
            permission = await createPermission.toPermission();
        } catch (error) {
            throw error;
        }

        permission = await this.permissionRepository.save(permission);
        permission = await this.permissionRepository.findOne(permission, { relations: ['principal'] });

        return new ResponsePermission(permission);
    }
    /*
    @Put('/:id')
    @ResponseSchema(ResponseRunnerTeam)
    @ResponseSchema(RunnerTeamNotFoundError, { statusCode: 404 })
    @ResponseSchema(RunnerTeamIdsNotMatchingError, { statusCode: 406 })
    @OpenAPI({ description: "Update a runnerTeam object (id can't be changed)." })
    async put(@Param('id') id: number, @Body({ validate: true }) runnerTeam: UpdateRunnerTeam) {
        let oldRunnerTeam = await this.runnerTeamRepository.findOne({ id: id }, { relations: ['parentGroup', 'contact'] });

        if (!oldRunnerTeam) {
            throw new RunnerTeamNotFoundError();
        }

        if (oldRunnerTeam.id != runnerTeam.id) {
            throw new RunnerTeamIdsNotMatchingError();
        }

        await this.runnerTeamRepository.update(oldRunnerTeam, await runnerTeam.toRunnerTeam());

        return new ResponseRunnerTeam(await this.runnerTeamRepository.findOne({ id: runnerTeam.id }, { relations: ['parentGroup', 'contact'] }));
    }

    @Delete('/:id')
    @ResponseSchema(ResponseRunnerTeam)
    @ResponseSchema(ResponseEmpty, { statusCode: 204 })
    @ResponseSchema(RunnerTeamHasRunnersError, { statusCode: 406 })
    @OnUndefined(204)
    @OpenAPI({ description: 'Delete a specified runnerTeam (if it exists).' })
    async remove(@Param("id") id: number, @QueryParam("force") force: boolean) {
        let team = await this.runnerTeamRepository.findOne({ id: id });
        if (!team) { return null; }
        let runnerTeam = await this.runnerTeamRepository.findOne(team, { relations: ['parentGroup', 'contact', 'runners'] });

        if (!force) {
            if (runnerTeam.runners.length != 0) {
                throw new RunnerTeamHasRunnersError();
            }
        }
        const runnerController = new RunnerController()
        for (let runner of runnerTeam.runners) {
            await runnerController.remove(runner.id, true);
        }

        const responseTeam = new ResponseRunnerTeam(runnerTeam);
        await this.runnerTeamRepository.delete(team);
        return responseTeam;
    }
    */
}
